<?php

/**
 * @file
 *
 * Support for migrating posts and pages from a Cultura Exchange site into host.
 */

/**
 * Implements MigrateSource to handle migrating items from Cultura Exchange XML.
 */
class CulturaExchangeItemSource extends MigrateSourceXML {

  /**
   * List of available source fields.
   *
   * @var array
   */
  protected $fields = array(
    'title' => 'Item title',
    'link' => 'URL of the item',
    'answer_first' => 'Column of answers for first language',
    'answer_second' => 'Column of answers for second language',
    'body' => 'Body text for the item',
    'summary' => 'Teaser for the item',
    'nid' => 'Unique ID of the item within the blog',
    'post_date' => 'Date posted (author\s timezone?)',
  );

  /**
   * Simple initialization.
   *
   */
  public function __construct($filename, $cache_key) {
    $source_options = array(
      'reader_class' => 'MigrateXMLReader',
      'cache_counts' => TRUE,
      'cache_key' => $cache_key,
    );
    parent::__construct($filename, '/site_export/nodes/node', 'nid',
      $this->fields, $source_options);
  }

  /**
   * Return a count of all available source records.
   */
  public function computeCount() {
    $count = 0;
    foreach ($this->sourceUrls as $url) {
      $reader = new $this->readerClass($url, $this->elementQuery, $this->idQuery);
      foreach ($reader as $element) {
        $count++;
      }
    }

    return $count;
  }
}

/**
 * Implementation of CulturaExchangeMigration for discussion nodes.
 */
class CulturaExchangeNode extends CulturaExchangeMigration {

  public function __construct(array $arguments = array()) {
    if (!isset($arguments['bundle'])) {
      $arguments['bundle'] = CULTURA_DISCUSSION_NODE_TYPE;
    }
    parent::__construct($arguments);

    // Drupal content type (bundle)
    $bundle = $arguments['bundle'];

    // Node ID (nid) is unique on a given 'live' Cultura Exchange site.
    $this->map = new MigrateSQLMap($this->machineName,
      array(
        'nid' => array(
          'type' => 'int',
          'not null' => TRUE,
          'unsigned' => TRUE,
          'description' => 'Source node ID',
        )
      ),
      MigrateDestinationNode::getKeySchema()
    );

    // Construct the source objects.
    $this->source = new CulturaExchangeItemSource($this->xmlFile, $this->machineName);
    $this->destination = new MigrateDestinationNode($bundle);

    // Add field mappings.
    $this->addFieldMapping('title', 'title')->xpath('title');
    $this->addFieldMapping('created', 'post_date')->xpath('post_date');
    $this->addFieldMapping('changed', 'post_date')->xpath('post_date');
    $uid = ($GLOBALS['user']->uid) ? $GLOBALS['user']->uid : 1;
    $this->addFieldMapping('uid')->defaultValue($uid);
    $this->addFieldMapping('cultura_questionnaire_prompts')->defaultValue('TRUE');
    $this->addFieldMapping('cultura_questionnaire_prompts:first', 'prompts_first')->xpath('prompts_first');
    $this->addFieldMapping('cultura_questionnaire_prompts:second', 'prompts_second')->xpath('prompts_second');
    $this->addFieldMapping('cultura_discussion_answers')->defaultValue('TRUE');
    $this->addFieldMapping('cultura_discussion_answers:first', 'answers_first')->xpath('answers_first');
    $this->addFieldMapping('cultura_discussion_answers:second', 'answers_second')->xpath('answers_second');
    $text_format = variable_get('culturaexchange_migrate_text_format', 'filtered_html');
    $this->addFieldMapping('cultura_question_type', 'question_type')->xpath('question_type');
    $this->createTermIfNeeded('cultura_year', CULTURA_YEAR);
    $this->addFieldMapping('cultura_year')->defaultValue($this->site->cultura_year);
    $this->createTermIfNeeded('cultura_semester', CULTURA_SEMESTER);
    $this->addFieldMapping('cultura_semester')->defaultValue($this->site->cultura_semester);
    $this->addFieldMapping('cultura_semester:ignore_case')->defaultValue(TRUE);
    $this->createTermIfNeeded('host_school', CULTURA_SCHOOLS);
    $this->addFieldMapping('cultura_host_school')->defaultValue($this->site->host_school);
    $this->addFieldMapping('cultura_host_school:ignore_case')->defaultValue(TRUE);
    $this->createTermIfNeeded('guest_school', CULTURA_SCHOOLS);
    $this->addFieldMapping('cultura_guest_school')->defaultValue($this->site->guest_school);
    $this->addFieldMapping('cultura_guest_school:ignore_case')->defaultValue(TRUE);
    $this->createTermIfNeeded('host_language', CULTURA_LANGUAGES);
    $this->addFieldMapping('cultura_host_language')->defaultValue($this->site->host_language);
    $this->addFieldMapping('cultura_host_language:ignore_case')->defaultValue(TRUE);
    $this->createTermIfNeeded('guest_language', CULTURA_LANGUAGES);
    $this->addFieldMapping('cultura_guest_language')->defaultValue($this->site->guest_language);
    $this->addFieldMapping('cultura_guest_language:ignore_case')->defaultValue(TRUE);

    // Unmapped destination fields
    $this->addUnmigratedDestinations(array(
      'is_new', 'revision', 'language', 'sticky', 'promote', 'revision_uid', 'log', 'tnid', 'status'));

  }

  /**
   * Create taxonomy terms for meta vocabularies if needed.
   *
   * @param string $key
   *   The site key that hold's the term's value.
   * @param string $vocab
   *   The machine name of the vocabulary to check and create the term in.
   */
  protected function createTermIfNeeded($key, $vocab) {
    if (!taxonomy_get_term_by_name($this->site->$key, $vocab)) {
      $term = (object) array(
        'name' => $this->site->$key,
        'vid' => taxonomy_vocabulary_machine_name_load($vocab)->vid,
      );
      taxonomy_term_save($term);
    }
  }
}
