<?php

/**
 * @file
 *
 * Web UI for migrating Cultura Exchange to Drupal.
 *
 * See https://www.drupal.org/project/cultura for the setup of a source
 * Cultura Exchange.
 */

/**
 * Menu callback: Returns a page for importing a Cultura Exchange into Drupal.
 */
function culturaexchange_migrate_import() {
  drupal_set_title(t('Cultura Exchange migration'));
  return drupal_get_form('culturaexchange_migrate_import_form');
}

/**
 * Form for specifying where to obtain the Cultura Exchange content.
 */
function culturaexchange_migrate_import_form($form, &$form_state) {
  // If an uploaded file exceeds post_max_size, the validate and submit functions
  // never get called (as they will if upload_max_filesize is exceeded but
  // post_max_size isn't). We can detect this case by the presence of an error.
  if ($error = error_get_last()) {
    drupal_set_message(t('File upload failed: !error. Please make sure the file
      you\'re trying to upload is not larger than !limit',
      array('!error' => $error['message'], '!limit' => format_size(file_upload_max_size()))));
  }

  // Make sure we have a private directory configured
  if (!variable_get('culturaexchange_migrate_private_path', '') &&
      !variable_get('file_private_path', '')) {
    $message = t('A private file system path must be <a href="@config">configured</a>
      to securely store your uploaded Cultura Exchange file.',
      array('@config' => url('admin/config/media/file-system')));
    $form['unconfigured'] = array(
      '#prefix' => '<div>',
      '#markup' => $message,
      '#suffix' => '</div>',
    );
    return $form;
  }

  $form['overview'] = array(
    '#prefix' => '<div>',
    '#markup' => t('Import content from a Cultura Exchange site.'),
    '#suffix' => '</div>',
  );

  $form['source']['source_select'] = array(
    '#type' => 'radios',
    '#options' => array(1 => t('Import from a URL'), 0 => t('Import from a file')),
    '#default_value' => 1,
    '#attributes' => array('class' => array('container-inline')),
    // Add some space between the two radio buttons.
    '#attached' => array('css' => array('#edit-source-select .form-item-source-select {padding-right: 1em;}' => array('type' => 'inline'))),
  );

  $form['#attributes'] = array('enctype' => 'multipart/form-data');
  $form['source']['xml_file'] = array(
    '#type' => 'file',
    '#title' => t('Cultura Exchange exported file (XML)'),
    '#description' => t('Select an exported Cultura Exchange file. Maximum file size is !size.',
      array('!size' => format_size(file_upload_max_size()))),
    '#states' => array(
      'visible' => array(
        ':input[name="source_select"]' => array('value' => 0),
      ),
    ),
  );

  $form['source']['credentials']['domain'] = array(
    '#type' => 'textfield',
    '#title' => t('Cultura Exchange site URL'),
    '#description' => t('Enter the URL of the exchange you want to import (e.g. http://culturaexchange.example.edu/a1b2c3d4e5f6).'),
    '#states' => array(
      'visible' => array(
        ':input[name="source_select"]' => array('value' => 1),
      ),
    ),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Import Cultura Exchange'),
  );

  return $form;
}

/**
 * Submit callback for the Cultura Exchange import form.
 */
function culturaexchange_migrate_import_form_submit($form, &$form_state) {
  $values = $form_state['values'];

  if ($_FILES['files']['error']['xml_file'] && empty($values['domain'])) {
    form_set_error('xml_file', t('The file could not be uploaded, most likely
      because the file size exceeds the configured limit of !filesize',
      array('!filesize' => format_size(file_upload_max_size()))));
    return;
  }
  $directory = 'culturaexchange://';
  if (!file_prepare_directory($directory, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
    form_set_error('xml_file',
      t('Could not prepare directory %directory - make sure your Drupal files directory exists and is writable',
        array('%directory' => $directory)));
    return;
  }

  $tmpfile = $_FILES['files']['tmp_name']['xml_file'];
  // Note that for either path we take here, $tmpfile will be the name of
  // the uploaded/retrieved file, and $destination will be the name of the
  // desired final destination.
  if ($tmpfile) {
    // Handle uploaded file
    $filename = $_FILES['files']['name']['xml_file'];
    $destination = $directory . str_replace(' ', '%20', $filename);
  }
  else {
    // Import the Cultura Exchange directly
    $site_url = trim($values['domain']);
    $destination = $directory . '/' . str_replace(array('/', ':'), array('_', ''), $site_url);
    $tmpfile = tempnam(sys_get_temp_dir(), 'ce_');
    $fp = fopen($tmpfile, 'w');
    if ($fp) {
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      curl_setopt($ch, CURLOPT_FILE, $fp);
      curl_setopt($ch, CURLOPT_URL, $site_url);
      curl_exec($ch);
      curl_close($ch);
      fclose($fp);
    }
    else {
      form_set_error('credentials][domain', t('Could not create destination file !filename',
        array('!filename' => $tmpfile)));
      return;
    }
  }

  CulturaExchangeSite::preprocessFile($tmpfile, $destination);

  try {
    $site = culturaexchange_migrate_site($destination);
  }
  catch (Exception $e) {
    form_set_error('xml_file', $e->getMessage());
    file_unmanaged_delete($destination);
    return;
  }

  // Import each migration in order, until done or time is running out
  if (variable_get('culturaexchange_migrate_import_method', 0) == 0) {
    $operations = array();
    foreach ($site->migrations() as $migration) {
      $operations[] = array(
        'migrate_ui_batch', array(
          'import', $migration->getMachineName(),
          array('unit' => 'items', 'value' => ''), FALSE
        )
      );
    }

    $batch = array(
      'operations' => $operations,
      'title' => t('Cultura Exchange import in progress'),
      'file' => drupal_get_path('module', 'migrate_ui') .
                    '/migrate_ui.pages.inc',
      'init_message' => t('Starting import process'),
      'progress_message' => t(''),
      'error_message' => t('An error occurred. Some or all of the import processing has failed.'),
      'finished' => 'migrate_ui_batch_finish',
    );
    batch_set($batch);
  }
  else {
    $spawned = FALSE;
    $force_drush = variable_get('culturaexchange_migrate_force_drush', FALSE);
    // Set up informational messages depending on Drush configuration.
    if ($force_drush) {
      $migration_in_progress = t('Your site is being imported. You will receive an email when the import is complete.');
    }
    else {
      $migration_in_progress = t('The site is too large to completely import immediately.
        The rest of the import will be run in the background and you will receive an email
        when it is complete.');
    }
    foreach ($site->migrations() as $migration) {
      if ($force_drush || (($result = $migration->processImport()) && $result == MigrationBase::RESULT_INCOMPLETE)) {
        $drush = variable_get('culturaexchange_migrate_drush', '');
        if (!$drush) {
          $message = t('The site was too large to import through the browser - please
              <a href="@config">configure drush</a> so the import process may be
              run in the background.',
            array('@config' => url('admin/content/culturaexchaneg')));
          form_set_error('xml_file', $message);
          break;
        }
        drupal_set_message($migration_in_progress);
        $uri = 'http://' . $_SERVER['HTTP_HOST'];
        $log_file = '/tmp/' . $filename . '.import.log';
        $uid = $GLOBALS['user']->uid;
        $command = "$drush --user=$uid --uri=$uri culturaexchange-migrate-import --root=" .
                   DRUPAL_ROOT . " $destination >$log_file 2>&1 &";
        exec($command);
        $spawned = TRUE;
        break;
      }
    }
    if (!$spawned) {
      drupal_set_message(t('File %filename successfully uploaded and imported',
        array('%filename' => $filename)));
    }
  }
  $form_state['redirect'] = 'admin/content/culturaexchange/review';
}

/**
 * Menu callback: Returns a page for reviewing Cultura Exchange migrations.
 */
function culturaexchange_migrate_review() {
  drupal_set_title(t('Cultura Exchange site migrations'));
  return drupal_get_form('culturaexchange_migrate_review_form');
}

/**
 * Form for reviewing Cultura Exchange migrations.
 */
function culturaexchange_migrate_review_form($form, &$form_state) {
  if (isset($form_state['values']['operation']) &&
      ($form_state['values']['operation'] == 'rollback' || $form_state['values']['operation'] == 'clear')) {
    return culturaexchange_migrate_rollback_confirm($form, $form_state, array_filter($form_state['values']['sites']));
  }
  $form['overview'] = array(
    '#prefix' => '<p>',
    '#markup' => t('The following Cultura Exchange sites have been imported into
      your host Cultura Drupal site. For each component of the site, the number
      of imported items is displayed.'),
    '#suffix' => '</p>',
  );

  $header = array(
    'site_url' => array('data' => t('Site URL')),
    'status' => array('data' => t('Status')),
  );
  $header += array(
    'CulturaExchangeNode' => array('data' => t('Nodes')),
  );
  if (module_exists('comment')) {
    $header += array(
      'CulturaExchangeComment' => array('data' => t('Comments')),
    );
  }
  $rows = array();
  $sites = CulturaExchangeSite::sites();
  // $sites = call_user_func(array(culturaexchange_migrate_site_class(), 'sites'));
  foreach ($sites as $site) {
    $classes = array_flip($site->migrationClasses());
    $row['site_url'] = $site->getSiteUrl();
    $row['status'] = t('Complete');
    foreach ($site->migrations() as $migration) {
      $class = $classes[get_class($migration)];
      $row[$class] = $migration->importedCount();
      $status = $migration->getStatus();
      if ($status == MigrationBase::STATUS_IMPORTING) {
        $row['status'] = t('Importing');
      }
      elseif ($status == MigrationBase::STATUS_ROLLING_BACK) {
        $row['status'] = t('Deleting');
      }
    }
    $rows[$site->getSiteUrl()] = $row;
  }

  $form['sites'] = array(
    '#type' => 'tableselect',
    '#header' => $header,
    '#options' => $rows,
    '#empty' => t('No Cultura Exchange sites have been imported.'),
  );

  // Build the 'Update options' form.
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Import rollback options'),
  );
  $form['options']['description'] = array(
    '#prefix' => '<div>',
    '#markup' => t('The Cultura Exchange import process tracks how the content
      maps to your host site so you can roll back the import process and restore
      your Drupal site to its original state. After you’ve finished the import,
      you can remove the additional rollback data.'),
    '#postfix' => '</div>',
  );
  $form['options']['operation'] = array(
    '#type' => 'radios',
    '#title' => t('Operation'),
    '#title_display' => 'invisible',
    '#options' => array(
      'rollback' => t('Roll back import'),
      'clear' => t('Delete tracking'),
    ),
  );
  $form['options']['operation']['rollback'] = array(
    '#description' => t('Restores your Drupal site to its pre-Cultura-Exchange-site import state. All imported content and rollback information will be deleted.'),
  );
  $form['options']['operation']['clear'] = array(
    '#description' => t('Removes the rollback information that allows you to roll back your import, and retains all imported content.'),
  );
  $form['options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
    '#validate' => array('culturaexchange_migrate_review_validate'),
    '#submit' => array('culturaexchange_migrate_review_submit'),
  );

  return $form;
}

/**
 * Validate callback for the Cultura Exchange review form.
 */
function culturaexchange_migrate_review_validate($form, &$form_state) {
  // Error if there are no items to select.
  if (!is_array($form_state['values']['sites']) || !count(array_filter($form_state['values']['sites']))) {
    form_set_error('', t('No items selected.'));
  }
}

/**
 * Submit callback for the Cultura Exchange review form.
 */
function culturaexchange_migrate_review_submit($form, &$form_state) {
  // We need to rebuild the form to go to a second step (confirm site rollback)
  $form_state['rebuild'] = TRUE;
}

function culturaexchange_migrate_rollback_confirm($form, &$form_state, $sites) {
  $operation = $form_state['values']['operation'];

  $form['sites'] = array('#prefix' => '<ul>', '#suffix' => '</ul>', '#tree' => TRUE);
  // array_filter returns only elements with TRUE values
  foreach ($sites as $site_url) {
    $form['sites'][$site_url] = array(
      '#type' => 'hidden',
      '#value' => $site_url,
      '#prefix' => '<li>',
      '#suffix' => check_plain($site_url) . "</li>\n",
    );
  }
  $form['operation'] = array('#type' => 'hidden', '#value' => $operation);
  $form['#submit'][] = 'culturaexchange_migrate_rollback_confirm_submit';
  if ($operation == 'rollback') {
    $confirm_question = format_plural(count($sites),
                                    'Are you sure you want to remove all imported content for this site?',
                                    'Are you sure you want to remove all imported content for these sites?');
  }
  else {
    $confirm_question = format_plural(count($sites),
                                    'Are you sure you want to remove all migration bookkeeping for this site?',
                                    'Are you sure you want to remove all migration bookkeeping for these sites?');
  }
  return confirm_form($form,
                    $confirm_question,
                    'admin/content/culturaexchange/review', t('This action cannot be undone.'),
                    t('Delete'), t('Cancel'));
}

function culturaexchange_migrate_rollback_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    $operations = array();
    $sites = array_keys($form_state['values']['sites']);
    foreach ($sites as $site_url) {
      // Rollback migrations for this domain
      $filename = db_select('culturaexchange_migrate', 'ce')
                   ->fields('ce', array('filename'))
                   ->condition('site_url', $site_url)
                   ->execute()
                   ->fetchField();
      $site = culturaexchange_migrate_site($filename);
      $migrations = array_reverse($site->migrations());
      foreach ($migrations as $migration) {
        // Only rollback content for the rollback operation
        if ($form_state['values']['operation'] == 'rollback') {
          // If not currently idle, stop it before attempting rollback
          if ($migration->getStatus() != MigrationBase::STATUS_IDLE) {
            $migration->stopProcess();
            // Give it a little time to react to the stop request
            $count = 5;
            while ($count && $migration->getStatus() == MigrationBase::STATUS_STOPPING) {
              sleep(2);
              $count--;
            }
            if ($migration->getStatus() == MigrationBase::STATUS_STOPPING) {
              // At this point, assume it's stuck and reset the status so we can continue
              $migration->resetStatus();
            }
          }
          $operations[] = array('migrate_ui_batch',
                                array('rollback', $migration->getMachineName(),
                                      array('unit' => 'items', 'value' => '')));
        }
      }
    }
    $batch = array(
      'operations' => $operations,
      'title' => t('Cultura Exchange site rollback in progress'),
      'file' => drupal_get_path('module', 'migrate_ui') .
                    '/migrate_ui.pages.inc',
      'init_message' => t('Starting rollback process'),
      'progress_message' => t(''),
      'error_message' => t('An error occurred. Some or all of the rollback processing has failed.'),
      'finished' => 'migrate_ui_batch_finish',
    );
    batch_set($batch);
    $operations = array();
    $operations[] = array('culturaexchange_migrate_batch_clear', array($sites));
    $batch = array(
      'operations' => $operations,
      'title' => t('Cultura Exchange rollback in progress'),
      'file' => drupal_get_path('module', 'culturaexchange_migrate') .
                    '/culturaexchange_migrate.pages.inc',
      'init_message' => t('Clearing housekeeping'),
      'progress_message' => t(''),
      'error_message' => t('An error occurred. Some or all of the rollback processing has failed.'),
      'finished' => 'migrate_ui_batch_finish',
    );
    batch_set($batch);
  }
}

/**
 * Forget about a site migration (without deleting imported content).
 */
function culturaexchange_migrate_batch_clear($sites, &$context) {
  foreach ($sites as $site_url) {
    $filename = db_select('culturaexchange_migrate', 'ce')
                 ->fields('ce', array('filename'))
                 ->condition('site_url', $site_url)
                 ->execute()
                 ->fetchField();
    $site = culturaexchange_migrate_site($filename);
    $migrations = $site->migrations();
    foreach ($migrations as $migration) {
      Migration::deregisterMigration($migration->getMachineName());
    }
    // Clear culturaexchange_migrate table entry
    db_delete('culturaexchange_migrate')
      ->condition('site_url', $site_url)
      ->execute();

    // Delete XML file
    file_unmanaged_delete($filename);

  }
  $context['finished'] = 1;
  $context['results'][]['message'] = format_plural(count($sites), 'Removed migration information for Cultura Exchange site.',
    'Removed migration information for @count Cultura Exchange sites.');
}
